<?php
/**
 *
 * @category        modules
 * @package         output_filter
 * @author          Christian Sommer, WB-Project, Werner v.d. Decken
 * @copyright       2013, WebsiteBaker Org. e.V.
 * @link            http://www.websitebaker.org/
 * @license         http://www.gnu.org/licenses/gpl.html
 * @platform        WebsiteBaker 2.8.4
 * @requirements    PHP 5.2.2 and higher
 * @version         $Id$
 * @filesource      $HeadURL$
 * @lastmodified    $Date$
 *
 */
/* -------------------------------------------------------- */
// Must include code to stop this file being accessed directly
require_once( dirname(dirname(dirname(__FILE__))).'/framework/globalExceptionHandler.php');
if(!defined('WB_PATH')) { throw new IllegalFileException(); }
/* -------------------------------------------------------- */

/* ************************************************************************** */
/**
 * execute the frontend output filter to modify email addresses
 * @param string actual content
 * @return string modified content
 */
	function executeFrontendOutputFilter($content) {
		// get output filter settings from database
		$filter_settings = getOutputFilterSettings();
		$sFilterDirectory = str_replace('\\', '/', dirname(__FILE__)).'/filters/';
		$output_filter_mode = 0;
		$output_filter_mode |= ($filter_settings['email_filter'] * pow(2, 0));  // n | 2^0
		$output_filter_mode |= ($filter_settings['mailto_filter'] * pow(2, 1)); // n | 2^1
		define('OUTPUT_FILTER_AT_REPLACEMENT', $filter_settings['at_replacement']);
		define('OUTPUT_FILTER_DOT_REPLACEMENT', $filter_settings['dot_replacement']);

/* ### filter type: replace MediaRel placeholder ############################ */
		if (file_exists($sFilterDirectory.'filterMediaRel.php')) {
			require_once($sFilterDirectory.'filterMediaRel.php');
			$content = doFilterMediaRel($content);
		}
/* ### filter type: change [wblinkxx] into real URLs ######################## */
		if (file_exists($sFilterDirectory.'filterWbLink.php')) {
			require_once($sFilterDirectory.'filterWbLink.php');
			$content = doFilterWbLink($content);
		}
/* ### filter type: execute droplets filter ################################# */
		if (file_exists($sFilterDirectory.'filterDroplets.php')) {
			require_once($sFilterDirectory.'filterDroplets.php');
			$content = doFilterDroplets($content);
		}
/* ### filter type: protect email addresses ################################# */
		if( ($output_filter_mode & pow(2, 0)) || ($output_filter_mode & pow(2, 1)) ) {
			if (file_exists($sFilterDirectory.'filterEmail.php')) {
				require_once($sFilterDirectory.'filterEmail.php');
				$content = doFilterEmail($content, $output_filter_mode);
			}
		}
/* ### filter type: replace MediaRel placeholder ############################ */
		if (file_exists($sFilterDirectory.'filterMediaRel.php')) {
			require_once($sFilterDirectory.'filterMediaRel.php');
			$content = doFilterMediaRel($content);
		}
/* ### filter type: change [wblinkxx] into real URLs ######################## */
		if (file_exists($sFilterDirectory.'filterWbLink.php')) {
			require_once($sFilterDirectory.'filterWbLink.php');
			$content = doFilterWbLink($content);
		}
/* ### filter type: full qualified URLs to relative URLs##################### */
        if($filter_settings['sys_rel'] == 1){
			if (file_exists($sFilterDirectory.'filterRelUrl.php')) {
				require_once($sFilterDirectory.'filterRelUrl.php');
				$content = doFilterRelUrl($content);
			}
		}
/* ### filter type: moves css definitions from <body> into <head> ########### */
		if (file_exists($sFilterDirectory.'filterCssToHead.php')) {
			require_once($sFilterDirectory.'filterCssToHead.php');
			$content = doFilterCssToHead($content);
		}

/* ### end of filters ####################################################### */
		return $content;
	}
/* ************************************************************************** */
/**
 * function to read the current filter settings
 * @global object $database
 * @global object $admin
 * @param void
 * @return array contains all settings
 */
	function getOutputFilterSettings() {
		global $database, $admin;
	// set default values
		$settings = array(
			'sys_rel'         => 0,
			'email_filter'    => 0,
			'mailto_filter'   => 0,
			'at_replacement'  => '(at)',
			'dot_replacement' => '(dot)'
		);
	// be sure field 'sys_rel' is in table
		$database->field_add( TABLE_PREFIX.'mod_output_filter', 'sys_rel', 'INT NOT NULL DEFAULT \'0\' FIRST');
	// request settings from database
		$sql = 'SELECT * FROM `'.TABLE_PREFIX.'mod_output_filter` ';
		if(($res = $database->query($sql))) {
			if(($rec = $res->fetchRow())) {
				$settings = $rec;
				$settings['at_replacement']  = $admin->strip_slashes($settings['at_replacement']);
				$settings['dot_replacement'] = $admin->strip_slashes($settings['dot_replacement']);
			}
		}
	// return array with filter settings
		return $settings;
	}
/* ************************************************************************** */
